@@ -391,3 +391,10 @@ class Database(object):
         command = SON([("$eval", code), ("args", list(args))])
         result = self._command(command)
         return result.get("retval", None)
+
+    def __call__(self):
+        """This is only here so that some API misusages are easier to debug.
+        """
+        raise TypeError("'Database' object is not callable. If you meant to "
+                        "call the '%s' method on a 'Collection' object it is "
+                        "failing because no such method exists." % self.__name)
